import urllib.request
class entSftpFile:
    # CREATE TABLE `sftp_files` (
    #   `id` int(10) NOT NULL AUTO_INCREMENT,
    #   `filename` varchar(20) COLLATE utf8_bin NOT NULL,
    #   `url` varchar(200) COLLATE utf8_bin NOT NULL,
    #   `stud` varchar(100) COLLATE utf8_bin NOT NULL,
    #   `uptime` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    #   PRIMARY KEY (`id`)
    # ) ENGINE=InnoDB AUTO_INCREMENT=482 DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
    filename = ""
    url = ""
    stud = ""

    def __init__(self,filename,url,stud) -> None:
        super().__init__()
        self.filename=filename
        url_encode = urllib.request.quote(url, safe='/:?=&', encoding='utf-8')
        self.url=url_encode
        # url解码
        # url_decode = urllib.request.unquote(url_encode)

        self.stud=stud





from dataclasses import dataclass
from datetime import datetime

from bgutils.spider.BaseSpider import BaseSpider

@dataclass
class SpiderCourse(BaseSpider):
    title: str #课程标题
    author: str #书作者
    link: str #课程链接
    desc: str #课程描述
    tags: str #标签
    price: str #课程单价
    pic: str #课程图片

    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username  #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "course_data"  #课程题材
        self.rawurl= rawurl #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充


from dataclasses import dataclass
from datetime import datetime

from bgutils.spider.BaseSpider import BaseSpider

@dataclass
class SpiderGames(BaseSpider):
    id: str #游戏ID
    title: str #游戏标题
    playlink: str #游戏地址
    introduce: str #游戏介绍
    pic : str #游戏图片地址

    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "games_data" #采集题材
        self.rawurl= rawurl #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充




from dataclasses import dataclass
from datetime import datetime

from bgutils.spider.BaseSpider import BaseSpider

@dataclass
class SpiderMeishi(BaseSpider):
    uid: str #商品ID
    username: str #作者
    title: str #商品标题
    mainingredient: str #原材料
    dateline :str #日期
    subject:str #主题
    url: str #商品url
    pic: str #商品的图片url,对应fcover

    # {
    #   "uid": "7796837",
    #   "username": "宸·羽",
    #   "id": "659229",
    #   "title": "香煎肉饼",
    #   "message": "",
    #   "mainingredient": "猪肉、木薯粉、虾皮、香菜、小葱、油、盐、蚝油、胡椒粉、五香粉、清水。",
    #   "dateline": "2024-5-28",
    #   "subject": "香煎肉饼",
    #   "fcover": "https://i3.meishichina.com/attachment/recipe/2024/05/29/2024052917169480832751.jpg?x-oss-process=style/c640",
    #   "cover": "https://i8.meishichina.com/attachment/recipe/2024/05/28/2024052817169106179691987796837.jpg?x-oss-process=style/f320x240",
    #   "mpic": "https://i3.meishichina.com/attachment/recipe/2024/05/29/2024052917169480832751.jpg?x-oss-process=style/c320",
    #   "tvpic": "attachment/recipe/2024/05/28/2024052817169106179691987796837.jpg",
    #   "mscover": "https://i3.meishichina.com/attachment/recipe/2024/05/29/2024052917169480832751.jpg?x-oss-process=style/c320",
    #   "path": "attachment/recipe/2024/05/28",
    #   "picname": "2024052817169106179691987796837.jpg",
    #   "collnum": "132",
    #   "viewnum": 0,
    #   "replynum": 0,
    #   "copyright": "1",
    #   "c320": "https://i3.meishichina.com/attachment/recipe/2024/05/29/2024052917169480832751.jpg?x-oss-process=style/c320",
    #   "avatar": "https://i5.meishichina.com/data/avatar/007/79/68/37_avatar_big.jpg?x-oss-process=style/c80",
    #   "likenum": 0,
    #   "isfav": 0,
    #   "islike": 0,
    #   "wapurl": "https://m.meishichina.com/recipe/all/elite/"
    # }

    def __init__(self, username, collector, rawurl, rawdata):
        self.username = username #采集者学号
        self.collector= collector #采集者姓名
        self.topic = "meishi_data" #采集题材
        self.rawurl= rawurl  #采集的原始url地址
        self.rawdata = rawdata #采集的原始数据，如记录行的json内容
        self.coll_time = datetime.now() #采集时间，实体初始方法自动填充



from datetime import datetime

from py4j.java_gateway import JavaGateway
from pyspark.sql import SparkSession
from pyspark import SparkContext
from pyspark.streaming import StreamingContext

class sparkUtil:
    def writehdfs(self):
        # conf = SparkConf().setMaster("local").setAppName('sparkUtil')
        # sc = SparkContext('local', 'test', conf=conf)
        spark = SparkSession.builder.getOrCreate()
        # spark = SparkSession.builder\
        #     .master("local") \
        #     .getOrCreate()
        # spark = SparkSession(conf).getOrCreate() #SparkSession(sc)

        # parquetFile = r"hdfs://host:port/Felix_test/test_data.parquet"
        parquetFile = r"hdfs://master:9870/user/limm/sparkSql/users.parquet"
        df = spark.read.parquet(parquetFile)
        print(df.first())

    def insHdsf(self):
        pass


    def mongo2hdfs(self, tblname, fdlist, where, pathtosave):
        print("tblname:" + tblname +",fdlist:" + fdlist+",where:"+where+",pathtosave:"+pathtosave)
        mongoUri = "mongodb+srv://home.hddly.cn:57017/pythondb"
        timetosleep = 10
        my_spark = SparkSession.builder \
            .appName("mongo2hdfs") \
            .master("local[*]") \
            .config("spark.jars.packages", "org.mongodb.spark:mongo-spark-connector_2.12:3.0.2") \
            .config("spark.mongodb.input.uri", mongoUri) \
            .config("spark.mongodb.output.uri", mongoUri) \
            .config("spark.network.timeout", "7200s") \
            .config("spark.executor.heartbeatInterval", "1200s") \
            .getOrCreate()


        # my_spark.conf.set('spark.sql.session.timeZone', 'UTC')
        tmptblname = "tmp_" + tblname
        isucc,ifail = 0,0
        timeDF = my_spark.sql("select  current_timestamp()")
        last_timestamp = timeDF.collect()[0][0]

        last_timestamp=last_timestamp + datetime.timedelta(minutes=-1)
        # relativedelta(minutes=-1)...strftime("%Y-%m-%d %H:%M:%S")
        last_time = format(last_timestamp, '%Y-%m-%d %H:%M:%S')
        print("last_time:"+last_time)
        # my_spark.read.format("mongo").option("collection", tblname).load() \
        #     .createOrReplaceTempView(tmptblname)
        my_spark.read.format("mongo").option("collection", tblname).load() \
            .createOrReplaceTempView(tmptblname)

        while True:
            datetime.sleep(timetosleep)
            timeDF = my_spark.sql("select  current_timestamp()")
            current_timestamp = timeDF.collect()[0][0]
            current_timestamp = current_timestamp + datetime.timedelta(minutes=-1)
            current_time = format(current_timestamp, '%Y-%m-%d %H:%M:%S')

            print("current_time:"+current_time)
            sql_str = "select %s from %s " % (fdlist, tmptblname)

            sqlDF = my_spark.sql(sql_str)

            sqldfwhere = sqlDF.where(where)

            sqldfilter1=sqldfwhere.filter(sqldfwhere["coll_time"]> last_timestamp)

            # sqldfilter = sqldfwhere.filter("coll_time > '" + last_time +"'")
            if (sqldfilter1.isEmpty()):
                print(current_time + ",Fail:" + str(ifail)+ ",Succ:" + str(isucc) )
                ifail+=1
            else:
                fileid = "music_data_" + format(current_timestamp, '%Y%m%d%H%M%S') + "_" + str(isucc).zfill(3)
                sqldfilter1.repartition(1).write.mode("overwrite").format("json") \
                    .save(pathtosave + fileid)
                print(current_time + ",Succ:" + str(isucc)+ ",Fail:" + str(ifail))
                isucc+=1

            last_time = current_time
            last_timestamp=current_timestamp
            print("current_time=>last_time:" + last_time)
            my_spark.stop

    def testhdfs(self):
        collector = "张三"
        tblname = "music_data_1"
        fdlist = "song_url,song_name,collector,coll_time"
        where = "collector='" + collector + "'"
        pathtosave = "hdfs://master:9864/user/myname/spark/"
        sparku.mongo2hdfs(tblname, fdlist, where, pathtosave)

    def dealstream(self):
        sc = SparkContext("local[2]", "NetworkWordCount")
        ssc = StreamingContext(sc, 1)
        java_gateway = JavaGateway.launch_gateway(die_on_exit=True)
        java_gateway.start_callback_server()
        lines = ssc.socketTextStream("slave1", 8888)
        words = lines.flatMap(lambda line: line.split(" "))
        pairs = words.map(lambda word: (word, 1))
        wordCounts = pairs.reduceByKey(lambda x, y: x + y)

        # Print the first ten elements of each RDD generated in this DStream to the console
        wordCounts.pprint()
        ssc.start()  # Start the computation
        # ssc.awaitTermination(10)  # Wait for the computation to terminate
        ssc.awaitTerminationOrTimeout(10)
        ssc.stop()

if __name__ == '__main__':
    sparku = sparkUtil()
    # sparku.writehdfs();
    sparku.dealstream();
    # sparku.testhdfs();



import csv


class CSVUtil:
    def __init__(self, file_path):
        self.file_path = file_path

    def read(self):
        data = []
        with open(self.file_path, 'r') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                data.append(row)
        return data

    def write(self, data):
        with open(self.file_path, 'w', newline='') as csvfile:
            writer = csv.writer(csvfile)
            for row in data:
                writer.writerow(row)